@@ -132,7 +132,7 @@ def scan_uart(port: str, measurement_obj=None, **kwargs):
                 _logger.debug("No update on a measurement object.")
             return uart_sensor(port)
         else:
-            _logger.debug(f"No known {uart_sensor} @ {port}")
+            _logger.debug(f"No known {uart_sensor.__name__} @ {port}")
 
     _logger.info(f"No known device @ {port}")
     return None
