@@ -25,7 +25,7 @@ class SDP8XX(I2CSensor):
     _i2c_freq = 10e3
     _SENSOR_ADDRESS = 0x25
     _crc_check_init = 0xFF
-    _temperature_scale_factor = 200  # 1/deg C
+    _temperature_scale_factor = 200  # 1/°C
     _temperature_unit = "deg C"
 
     _cmd_cont_meas_diff_pressure_avg = [0x36, 0x03]
@@ -54,8 +54,7 @@ class SDP8XX(I2CSensor):
         self.exists = self.sensor_exists()
 
     def sensor_exists(self):
-        """
-        test if sensor is plugged in and works proper
+        """Test if sensor is plugged in and works proper
         :return: [BOOLEAN] True if test was successful otherwise False
         """
         self.reset()
@@ -69,8 +68,7 @@ class SDP8XX(I2CSensor):
             return False
 
     def reset(self):
-        """
-        reseting the sensor
+        """Reseting the sensor
         :return: True for successful reset else False
         """
         try:
@@ -80,10 +78,10 @@ class SDP8XX(I2CSensor):
         except IOError:
             return False
 
-    def crc_check(self, ba):
+    def crc_check(self, ba: bytearray):
         """
-        :param ba: [BYTEARRAY] byte array with crc check sum every third entry
-        :return: Array of booleans indicated correct crc check sums
+        :param ba: byte array with n*3 entries. Two data bytes are followed by one crc check.
+        :return: list of booleans with len n indicating crc results
         """
         crc = [False] * int(len(ba) / 3)
         for i in range(0, len(ba), 3):
@@ -92,18 +90,26 @@ class SDP8XX(I2CSensor):
             )[1]
         return crc
 
+    def check_if_supported(self, ba):
+        if ba not in self.__supported_sensors.keys():
+            return False
+        return True
+
+    def get_raw_serial(self):
+        self.txrx(self._cmd_product_identifier[:2], readlen=0)
+        self.txrx(self._cmd_product_identifier[2:], readlen=0)
+        ba = self.mux.receive(self._SENSOR_ADDRESS, self.mux_port, readlength=18)
+        return ba
+
     def get_serial_number(self):
-        """
-        reads unique serial number of sensor
+        """Reads unique serial number of sensor
         :return: [STRING] serial number of the sensor
         """
         self.serial_number = None
         self.error = None
-        self.txrx(self._cmd_product_identifier[:2], readlen=0)
-        self.txrx(self._cmd_product_identifier[2:], readlen=0)
-        ba = self.mux.receive(self._SENSOR_ADDRESS, self.mux_port, readlength=18)
+        ba = self.get_raw_serial()
 
-        if ba is not None:
+        if ba is not None and len(ba) == 18:
 
             crc = self.crc_check(ba)
 
@@ -112,11 +118,16 @@ class SDP8XX(I2CSensor):
                 hex_str = ""
                 for i in [0, 1, 3, 4]:
                     hex_str += f"{ba[i]:02X}"
-                assert (
-                    hex_str[:-2] in self.__supported_sensors.keys()
-                ), f"Product {hex_str} not supported."
+
+                if not self.check_if_supported(hex_str[:-2]):
+                    self.error = Error().unsupported_sensor(self)
+                    return self.error
+
                 self.product_number = hex_str[:-2]
                 self.sensor_type = self.__name__
+            else:
+                self.error = Error().crc(self)
+                return self.error
 
             # Serial number
             if crc[2] and crc[3] and crc[4] and crc[5]:
@@ -134,64 +145,64 @@ class SDP8XX(I2CSensor):
             return self.error
 
     def prepare_measurement(self):
-        self.txrx(self._cmd_cont_meas_diff_pressure, readlen=0)
-        ba = self.txrx([], readlen=9)
-        if ba is not None:
+        self.start_continuous_measurement()
+        ba = self.get_raw_data()
+        if ba is not None and len(ba) == 9:
             self.crc_check(ba)
             self.scale_factor = ba[6] << 8 | ba[7]
+            return True
         else:
             self.mux.close_all_ports()
             self.reset()
             self.error = Error().read(self)
             return self.error
 
+    def start_continuous_measurement(self):
+        self.txrx(self._cmd_cont_meas_diff_pressure, readlen=0)
+
+    def get_raw_data(self):
+        return self.txrx([], readlen=9)
+
     def get_data(self):
         """
-        :param scale_factor: [INT] sensor specific parameter for the
-        calculation of the volume flow
-        :param offset: [INT] sensor specific parameter for the
-        calculation of the volume flow
-        :return: [DICT] data dictionary
+        :return: data dictionary
+        :rtype: dict
         """
         data = self.default_data()
-
-        ba = self.txrx([], readlen=9)
+        ba = self.get_raw_data()
 
         if ba is not None and len(ba) == 9:
             # Perform crc check.
             crc = self.crc_check(ba)
             if crc[0] and crc[1] and crc[2]:
 
-                # Get pressure (signed integer)
-                raw_dp = (ba[0] << 8) | ba[1]
-                if raw_dp > 32768:
-                    dp = round((raw_dp - 2 ** 16) / ((ba[6] << 8) | ba[7]), 3)
-                else:
-                    dp = round(raw_dp / ((ba[6] << 8) | ba[7]), 3)
+                k_dp = self.bytes_to_u16(ba[6], ba[7])  # 1 / Pa
+                if k_dp == 0:
+                    self._logger.warning("Differential pressure scale == 0")
+                    return Error().read(self)
 
-                # Get temperature
-                T = ((ba[3] << 8) | ba[4]) / self._temperature_scale_factor
-                T = round(T, 2)
-
-                self._logger.debug("dP" + str(dp) + "\t" + str(raw_dp))
+                # Calculate physical quantities
+                dp = self.getS16(ba[0], ba[1]) / k_dp  # Pa
+                T = self.getS16(ba[3], ba[4]) / self._temperature_scale_factor  # °C
 
                 # Setup return values
                 data["values"] = {}
                 data["values"]["temperature"] = {
-                    "value": T,
+                    "value": round(T, 2),
                     "unit": self._units["temperature"],
                 }
                 data["values"]["diff_pressure"] = {
-                    "value": dp,
+                    "value": round(dp, 3),
                     "unit": self._units["diff_pressure"],
                 }
 
-                # Set error to false
+                # Mark values as valid
                 data["error"] = False
 
             else:
                 self._logger.debug("CRC Error [%s, %s, %s]", crc)
-                data = Error().read(self)
+                self.error = Error().crc(self)
+                data = self.error
                 self.reset()
                 self.prepare_measurement()
         else:
