@@ -246,7 +246,7 @@ class SMGair2(UARTSensor):
         sn = self.txrx(msg)
         while 0x00 in sn:
             sn = sn[:-1]
-        return str(sn)
+        return sn.decode("utf-8")
 
     def txrx(self, msg):
         """
